--An attempt to make a "Newton's Cradle" simulation: http://en.wikipedia.org/wiki/Newton%27s_cradle

--A list of all the global variables
function global_variables()
	world = love.physics.newWorld(-650, -650, 650, 650)
	
	bodiesRec = {}
	shapesRec = {}
	bodiesCir = {}
	shapesCir = {}
	joints = {}
	
	gravityX = 0
	gravityY = 100
	pixelsPerMeter = 30 -- One circle is 2 meters diagonally
end

function love.load()
	global_variables()

	world:setGravity(gravityX, gravityY)
	world:setMeter(pixelsPerMeter)
	
	local numEls = 3
	local dist = 100
	local radius = 30
	local rotation = 0
	
	local margin = 75
	local xOffset = radius + dist
	local yOffset = 0
	
	for i = 1, numEls, 1 do
		rotation = math.pi / 2
		
		if i == 1 then
			rotation = math.pi
		end
		
		createCradleEl(margin + xOffset, margin + yOffset, dist, radius, rotation)
		
		xOffset = xOffset + radius * 2
	end
	
	love.graphics.setBackgroundColor(55, 188, 220)
	love.graphics.setMode(margin * 2 + dist + xOffset, 2 * margin + dist + radius, false, true, 0)
end


function love.update(dt)
	world:update(dt)
end

function love.draw()	
	love.graphics.setColor(0, 0, 0)
	for i = 1, #joints, 1 do
		love.graphics.line(joints[i]:getAnchors())
	end
	
	love.graphics.setColor(238, 241, 178)
	for i = 1, #shapesRec, 1 do
		drawRectangleShape(i)
	end
	
	love.graphics.setColor(160, 212, 138)
	for i = 1, #shapesCir, 1 do
		drawCircleShape(i)
	end
	
	love.graphics.setColor(0, 0, 0)
	for i = 1, #joints, 1 do
		local x1, y1, x2, y2 = joints[i]:getAnchors()
		love.graphics.circle("line", x1, y1, 3)
		love.graphics.circle("line", x2, y2, 3)
	end
end

--Create a cradle element, which consists of a stationary element and the circle 
--(and the joint between them)
function createCradleEl(x, y, distance, radius, rotation)
	local cirOffsetX = distance * math.cos(rotation)
	local cirOffsetY = distance * math.sin(rotation)
	
	local bodyRec
	local shapeRec
	local bodyCir
	local shapeCir
	local joint
	
	bodyRec = love.physics.newBody(world, x, y, 0, 0)
	shapeRec = love.physics.newRectangleShape(bodyRec, 0, 0, 20, 20, 0)
	
	bodyCir = love.physics.newBody(world, x + cirOffsetX, y + cirOffsetY, circleMass, 0)
	shapeCir = love.physics.newCircleShape(bodyCir, 0, 0, radius)
	bodyCir:setMassFromShapes()
	
	joint = love.physics.newDistanceJoint(bodyCir, bodyRec, x + cirOffsetX, y + cirOffsetY, x, y)
		
	table.insert(bodiesRec, bodyRec)
	table.insert(shapesRec, shapeRec)
	
	table.insert(bodiesCir, bodyCir)
	table.insert(shapesCir, shapeCir)
	
	table.insert(joints, joint)
end

--Draw a rectangle shape
function drawRectangleShape(id)
	local x1, y1, x2, y2, x3, y3, x4, y4 = shapesRec[id]:getBoundingBox()
	local boxwidth = x3 - x2
	local boxheight = y2 - y1
	
	love.graphics.rectangle("fill", bodiesRec[id]:getX() - boxwidth/2, bodiesRec[id]:getY() - 
		boxheight/2, boxwidth, boxheight)
end

--Draw a circle shape
function drawCircleShape(id)
	love.graphics.circle("fill", bodiesCir[id]:getX(), bodiesCir[id]:getY(), 
		shapesCir[id]:getRadius(), 32)
end